<?php
if ( !defined( 'ABSPATH' ) ) {
    exit;
} // Exit if accessed directly

/**
 * Member Class
 *
 * @class   YITH_WCBK_Booking_Helper
 * @package Yithemes
 * @since   1.0.0
 * @author  Yithemes
 *
 */
class YITH_WCBK_Booking_Helper {

    /**
     * Single instance of the class
     *
     * @var \YITH_WCBK_Booking_Helper
     */
    protected static $_instance;

    /**
     * the booking post type name
     *
     * @var string
     * @since 1.0.0
     */
    public $post_type_name;

    /**
     * Returns single instance of the class
     *
     * @return \YITH_WCBK_Booking_Helper
     * @since 1.0.0
     */
    public static function get_instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    /**
     * Constructor
     *
     *
     * @since  1.0.0
     * @author Leanza Francesco <leanzafrancesco@gmail.com>
     */
    public function __construct() {
        $this->post_type_name = YITH_WCBK_Post_Types::$booking;
    }

    /**
     * Count the number of booked booking of this product
     * in a specific period
     *
     * @param int $from timestamp
     * @param int $to timestamp
     *
     * @return int
     */
    public function count_booked_bookings_in_period( $product_id, $from, $to, $get_post_args = array() ) {
        $meta_query = array(
            'relation' => 'AND',
            array(
                'key'   => '_product_id',
                'value' => $product_id,
            ),
            array(
                'key'     => '_from',
                'value'   => $to,
                'compare' => '<'
            ),
            array(
                'key'     => '_to',
                'value'   => $from,
                'compare' => '>'
            ),
        );

        $active_booking_statuses = array(
            'bk-unpaid',
            'bk-paid',
            'bk-completed',
            'bk-confirmed',
        );

        $args = array(
            'post_status' => $active_booking_statuses,
            'meta_query'  => $meta_query,
        );

        $args = wp_parse_args( $get_post_args, $args );

        $bookings = $this->get_bookings( $args, 'posts' );

        return !!$bookings ? count( $bookings ) : 0;
    }

    /**
     * return count of bookings with specific status
     *
     * @param string|array $status
     *
     * @return int
     */
    public function count_booking_with_status( $status ) {
        $counter = 0;
        if ( !is_array( $status ) ) {
            $status = array( $status );
        }
        foreach ( $status as $s ) {
            if ( yith_wcbk_is_a_booking_status( $s ) ) {
                $counts = (array) wp_count_posts( $this->post_type_name );
                $counter += isset( $counts[ 'bk-' . $s ] ) ? absint( $counts[ 'bk-' . $s ] ) : 0;
            }
        }

        return $counter;
    }

    /**
     * Get all bookings by arguments
     *
     *
     * @param array  $args argument for get_posts
     * @param string $return the object type returned
     *
     * @since  1.0.0
     * @author Leanza Francesco <leanzafrancesco@gmail.com>
     * @return YITH_WCBK_Booking[]|bool
     */
    public function get_bookings( $args = array(), $return = 'bookings' ) {
        $all_booking_statuses = array_keys( yith_wcbk_get_booking_statuses() );

        foreach ( $all_booking_statuses as $key => $value ) {
            $all_booking_statuses[ $key ] = 'bk-' . $value;
        }

        $default_args = array(
            'post_type'      => $this->post_type_name,
            'post_status'    => $all_booking_statuses,
            'posts_per_page' => -1,
        );

        $args = wp_parse_args( $args, $default_args );

        $bookings = get_posts( $args );

        switch ( $return ) {
            case 'posts':
                return $bookings;
                break;
            case 'bookings':
            default:
                return $this->parse_bookings_from_posts( $bookings );
        }
    }

    /**
     * Get all bookings of a user
     *
     *
     * @param int    $user_id the id of the user
     *
     * @param string $return
     *
     * @return bool|YITH_WCBK_Booking[]
     * @since  1.0.0
     * @author Leanza Francesco <leanzafrancesco@gmail.com>
     */
    public function get_bookings_by_user( $user_id, $return = 'bookings' ) {
        if ( !$user_id ) {
            return false;
        }
        $args = array(
            'meta_key'   => '_user_id',
            'meta_value' => $user_id,
        );

        return $this->get_bookings( $args, $return );
    }

    /**
     * Get all bookings of a booking product
     *
     *
     * @param int    $product_id the id of the product
     *
     * @param string $return
     *
     * @return bool|YITH_WCBK_Booking[]
     * @since  1.0.0
     * @author Leanza Francesco <leanzafrancesco@gmail.com>
     */
    public function get_bookings_by_product( $product_id, $return = 'bookings' ) {
        if ( !$product_id ) {
            return false;
        }

        $args = array(
            'meta_key'   => '_product_id',
            'meta_value' => $product_id,
        );

        return $this->get_bookings( $args, $return );
    }

    /**
     * Get all bookings by meta_query
     *
     *
     * @param array  $meta_query the meta query
     * @param string $return the object type returned
     *
     * @since  1.0.0
     * @author Leanza Francesco <leanzafrancesco@gmail.com>
     * @return YITH_WCBK_Booking[]|bool
     */
    public function get_bookings_by_meta( $meta_query, $return = 'bookings' ) {
        $args = array(
            'meta_query' => $meta_query,
        );

        return $this->get_bookings( $args, $return );
    }

    /**
     * @param        $start
     * @param        $end
     * @param string $return
     *
     * @return bool|YITH_WCBK_Booking[]
     */
    public function get_bookings_in_time_range( $start, $end, $return = 'bookings' ) {
        $meta_query = array(
            'relation' => 'AND',
            array(
                'key'     => '_from',
                'value'   => $end,
                'compare' => '<='
            ),
            array(
                'key'     => '_to',
                'value'   => $start,
                'compare' => '>'
            ),
        );

        $args = array(
            'meta_query' => $meta_query,
            'meta_key'   => '_from',
            'orderby'    => 'meta_value',
            'order'      => 'ASC'
        );

        return $this->get_bookings( $args, $return );
    }

    /**
     * get bookings by order
     *
     * @param int $order_id the id of the order
     * @param int $order_item_id the id of the order item
     *
     * @access public
     * @since  1.0.0
     * @return bool|YITH_WCBK_Booking
     */
    public function get_bookings_by_order( $order_id, $order_item_id = '' ) {
        if ( !$order_id )
            return false;

        $meta_query = array(
            'relation' => 'AND',
            array(
                'key'   => '_order_id',
                'value' => $order_id,
            )
        );

        if ( !empty( $order_item_id ) ) {
            $meta_query[] = array(
                'key'   => '_order_item_id',
                'value' => $order_item_id,
            );
        }

        return $this->get_bookings_by_meta( $meta_query );
    }

    /**
     * parse posts and return array of YITH_WCBK_Booking
     *
     * @param WP_Post|WP_Post[] $booking_posts the posts
     *
     *
     * @since  1.0.0
     * @author Leanza Francesco <leanzafrancesco@gmail.com>
     * @return YITH_WCBK_Booking[]|bool
     */
    public function parse_bookings_from_posts( $booking_posts ) {
        if ( !empty( $booking_posts ) ) {
            $booking_posts = (array) $booking_posts;
            $bookings      = array();
            foreach ( $booking_posts as $post ) {
                $booking    = new YITH_WCBK_Booking( $post->ID );
                $bookings[] = $booking;
            }

            return $bookings;
        }

        return false;
    }

}

/**
 * Unique access to instance of YITH_WCBK_Booking_Helper class
 *
 * @return \YITH_WCBK_Booking_Helper
 */
function YITH_WCBK_Booking_Helper() {
    return YITH_WCBK_Booking_Helper::get_instance();
}
