<?php
/**
 * Main Booking class
 *
 * @author  Yithemes
 * @package YITH Booking for WooCommerce Premium
 * @version 1.0.0
 */


if ( !defined( 'YITH_WCBK' ) ) {
    exit;
} // Exit if accessed directly

if ( !class_exists( 'YITH_WCBK_Booking' ) ) {
    /**
     * YITH_WCBK_Booking
     *
     * @since 1.0.0
     */
    class YITH_WCBK_Booking {

        /**
         * id of booking
         *
         * @var int
         * @since 1.0.0
         */
        public $id;

        /**
         * post of booking
         *
         * @var WP_Post|bool
         * @since 1.0.0
         */
        public $post;

        /**
         * Constructor
         *
         * @param int   $booking the booking id
         * @param array $args array of meta for creating booking
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com>
         */
        public function __construct( $booking = 0, $args = array() ) {
            //the booking if $booking_id is defined
            if ( $booking ) {
                if ( is_numeric( $booking ) ) {
                    $this->id   = absint( $booking );
                    $this->post = get_post( $this->id );
                } elseif ( $booking instanceof YITH_WCBK_Booking ) {
                    $this->id   = absint( $booking->id );
                    $this->post = $booking->post;
                } elseif ( isset( $booking->ID ) ) {
                    $this->id   = absint( $booking->ID );
                    $this->post = $booking;
                }
            }

            //add a new booking if $args is passed
            if ( !$booking && !empty( $args ) ) {
                $this->create_booking( $args );
            }

        }

        /**
         * __get function.
         *
         * @param string $key
         *
         * @return mixed
         */
        public function __get( $key ) {

            if ( 'status' === $key ) {
                $this->status = get_post_status( $this->id );

                return $this->status;
            } elseif ( 'services' === $key ) {
                $this->services = wp_get_post_terms( $this->id, YITH_WCBK_Post_Types::$service_tax, array( 'fields' => 'ids' ) );

                return $this->services;
            }

            $value = get_post_meta( $this->id, '_' . $key, true );

            if ( !empty( $value ) ) {
                $this->$key = $value;
            }

            return $value;
        }

        /**
         * __isset function.
         *
         * @param string $key
         *
         * @return mixed
         */
        public function __isset( $key ) {
            if ( 'status' === $key ) {
                $this->status = get_post_status( $this->id );

                return true;
            } elseif ( 'services' === $key ) {
                $this->services = wp_get_post_terms( $this->id, YITH_WCBK_Post_Types::$service_tax, array( 'fields' => 'ids' ) );

                return true;
            }

            return metadata_exists( 'post', $this->id, '_' . $key );
        }

        /**
         * __set function.
         *
         * @param string $property
         * @param mixed  $value
         *
         * @return bool|int
         */
        public function set( $property, $value ) {
            if ( 'status' === $property ) {
                $this->status     = 'bk-' . $value;
                $update_post_data = array(
                    'ID'          => $this->id,
                    'post_status' => $this->status,
                );

                return wp_update_post( $update_post_data );
            }
            $this->$property = $value;

            return update_post_meta( $this->id, '_' . $property, $value );
        }

        /**
         * @param        $type
         * @param string $note
         */
        public function add_note( $type, $note = '' ) {
            YITH_WCBK()->notes->add_booking_note( $this->id, $type, $note );
        }

        /**
         * Add new membership
         *
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com
         * @return void
         */
        public function create_booking( $args ) {
            $booking_title = isset( $args[ 'title' ] ) ? $args[ 'title' ] : '';

            $booking_id = wp_insert_post( array(
                                              'post_status' => isset( $args[ 'status' ] ) ? $args[ 'status' ] : 'bk-unpaid',
                                              'post_type'   => YITH_WCBK_Post_Types::$booking,
                                              'post_title'  => $booking_title
                                          ) );

            if ( $booking_id ) {
                $this->id = $booking_id;
                /**
                 * @var WC_Product_Booking $product
                 */
                if ( isset( $args[ 'product_id' ] ) && $product = wc_get_product( absint( $args[ 'product_id' ] ) ) ) {
                    $args[ 'can_be_cancelled' ]   = $product->get_booking_prop( 'can_be_cancelled' ) === 'yes';
                    $args[ 'cancelled_duration' ] = $product->get_booking_prop( 'cancelled_duration' );
                    $args[ 'cancelled_unit' ]     = $product->get_booking_prop( 'cancelled_unit' );
                }

                $meta = wp_parse_args( $args, $this->get_default_meta_data() );
                $this->update_booking_meta( $meta );
                $this->populate();

                $this->add_note( 'new', __( 'Booking successfully created.', 'yith-booking-for-woocommerce' ) );

                do_action( 'yith_wcbk_booking_created', $this );

                WC()->mailer();
                do_action( 'yith_wcbk_new_booking', $booking_id );
            }
        }

        /**
         * @return array|null|object
         */
        public function get_notes() {
            return YITH_WCBK()->notes->get_booking_notes( $this->id );
        }

        /**
         * Get the title with id
         *
         * @return string
         */
        public function get_title() {
            return '#' . $this->id . ' ' . $this->title;
        }

        /**
         * Get the service names for current booking
         *
         * @return array
         */
        public function get_service_names( $show_hidden = true ) {
            $services = array();
            if ( !!$this->services ) {
                foreach ( $this->services as $service_id ) {
                    $service = yith_get_booking_service( $service_id );
                    if ( $service->is_valid() ) {
                        if ( $show_hidden || !$service->is_hidden() )
                            $services[] = $service->get_name();
                    }
                }
            }

            return $services;
        }

        /**
         * Get the duration of booking including duration unit
         */
        public function get_duration_html() {
            $duration            = $this->duration;
            $duration_unit_label = yith_wcbk_get_duration_unit_label( $this->duration_unit, $duration );

            return $duration . ' ' . $duration_unit_label;
        }

        /**
         * Generates a URL to view a booking from the my account page.
         *
         * @return string
         */
        public function get_view_booking_url() {
            $view_booking_endpoint = YITH_WCBK()->endpoints->get_endpoint( 'view-booking' );

            $view_booking_url = wc_get_endpoint_url( $view_booking_endpoint, $this->id, wc_get_page_permalink( 'myaccount' ) );

            return apply_filters( 'yith_wcbk_get_view_booking_url', $view_booking_url, $this );
        }

        /**
         * Generates a URL to cancel a booking from the my account page.
         *
         * @return string
         */
        public function get_cancel_booking_url() {
            $bookings_endpoint = YITH_WCBK()->endpoints->get_endpoint( 'bookings' );
            $bookings_url      = wc_get_endpoint_url( $bookings_endpoint, '', wc_get_page_permalink( 'myaccount' ) );
            $cancel_url        = add_query_arg( array( 'cancel-booking' => $this->id ), $bookings_url );

            return apply_filters( 'yith_wcbk_get_cancel_booking_url', $cancel_url, $this );
        }

        /**
         * Generates a URL to pay a booking from the my account page.
         *
         * @return string
         */
        public function get_confirmed_booking_payment_url() {
            $bookings_endpoint = YITH_WCBK()->endpoints->get_endpoint( 'bookings' );
            $bookings_url      = wc_get_endpoint_url( $bookings_endpoint, '', wc_get_page_permalink( 'myaccount' ) );
            $payment_url       = add_query_arg( array( 'pay-confirmed-booking' => $this->id ), $bookings_url );

            return apply_filters( 'yith_wcbk_get_confirmed_booking_payment_url', $payment_url, $this );
        }

        /**
         * Populate the booking
         *
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com
         * @return void
         */
        public function populate() {
            $this->post = get_post( $this->id );

            foreach ( $this->get_booking_meta() as $key => $value ) {
                $this->$key = $value;
            }

            do_action( 'yith_wcbk_booking_loaded', $this );
        }

        /**
         * Check if the booking is valid, controlling if this post exist
         *
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com
         * @return bool
         */
        public function is_valid() {
            return !empty( $this->post ) && YITH_WCBK_Post_Types::$booking === $this->post->post_type;
        }

        /**
         * Update post meta in booking
         *
         * @param array $meta the meta
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com
         * @return void
         */
        function update_booking_meta( $meta ) {
            foreach ( $meta as $key => $value ) {
                if ( $key !== 'services' ) {
                    update_post_meta( $this->id, '_' . $key, $value );
                } else {
                    wp_set_post_terms( $this->id, $value, YITH_WCBK_Post_Types::$service_tax );
                }
            }
        }

        /**
         * Fill the default metadata with the post meta stored in db
         *
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com
         * @return array
         */
        function get_booking_meta() {
            $meta = array();
            foreach ( $this->get_default_meta_data() as $key => $value ) {
                if ( $key !== 'services' ) {
                    $meta[ $key ] = get_post_meta( $this->id, '_' . $key, true );
                } else {
                    $meta[ $key ] = wp_get_post_terms( $this->id, YITH_WCBK_Post_Types::$service_tax, array( 'fields' => 'ids' ) );
                }
            }

            return $meta;
        }

        /**
         * Return an array of all custom fields booking
         *
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com
         * @return array
         */
        private function get_default_meta_data() {
            $default_meta_data = array(
                'product_id'         => 0,
                'title'              => '',
                'from'               => '',
                'to'                 => '',
                'duration'           => 1,
                'duration_unit'      => '',
                'persons'            => 1,
                'person_types'       => array(),
                'order_id'           => 0,
                'order_item_id'      => 0,
                'user_id'            => 0,
                'services'           => array(),
                'can_be_cancelled'   => false,
                'cancelled_duration' => 0,
                'cancelled_unit'     => 'month',
                'activities'         => array(),

            );

            return $default_meta_data;
        }

        /**
         * Get the person types HTML
         *
         * @return string
         */
        public function get_person_types_html() {
            $html = '';
            if ( !empty( $this->person_types ) ) {
                foreach ( $this->person_types as $person_type ) {
                    $id     = isset( $person_type[ 'id' ] ) ? $person_type[ 'id' ] : false;
                    $title  = isset( $person_type[ 'title' ] ) ? $person_type[ 'title' ] : false;
                    $number = isset( $person_type[ 'number' ] ) ? $person_type[ 'number' ] : false;

                    if ( $id === false || $title === false || !$number )
                        continue;

                    $person_type_title = get_the_title( $id );
                    $title             = !!$person_type_title ? $person_type_title : $title;
                    $html              .= "<strong>{$title}:</strong> {$number}";
                    $html              .= "<br />";
                }
            }

            return $html;
        }

        /**
         * Return the status
         *
         * @access public
         * @since  1.0.0
         *
         * @return string
         */
        public function get_status() {
            if ( !isset( $this->status ) )
                $this->status = get_post_status( $this->id );

            return apply_filters( 'yith_wcbk_booking_get_status', 'bk-' === substr( $this->status, 0, 3 ) ? substr( $this->status, 3 ) : $this->status, $this );
        }

        /**
         * Return string for status
         *
         * @access public
         * @since  1.0.0
         *
         * @return string
         */
        public function get_status_text() {
            $text = strtr( $this->get_status(), yith_wcbk_get_booking_statuses() );

            return $text;
        }

        /**
         * Return string for dates
         *
         * @param string $date_type the type of date : from | to
         *
         * @access public
         * @since  1.0.0
         *
         * @return string
         */
        public function get_formatted_date( $date_type ) {
            $units_without_time = array( 'month', 'day' );
            $with_time          = !in_array( $this->duration_unit, $units_without_time );

            $format = wc_date_format();
            $format .= $with_time ? ( ' ' . wc_time_format() ) : '';

            return apply_filters( 'yith_wcbk_booking_get_formatted_date', date_i18n( $format, $this->$date_type ), $date_type, $this );
        }

        /**
         * check if booking has person types
         *
         * @return bool
         */
        public function has_person_types() {
            return !empty( $this->person_types );
        }

        /**
         *
         * check if the booking can change status to $status
         *
         * @param $status
         *
         * @return bool
         */
        public function can_be( $status ) {

            $value = false;

            switch ( $status ) {
                case 'cancelled_by_user':
                    if ( !!$this->can_be_cancelled && $this->has_status( array( 'unpaid',
                                                                                'paid',
                                                                                'pending-confirm',
                                                                                'confirmed' ) )
                    ) {
                        $now              = strtotime( 'now' );
                        $last_cancel_date = YITH_WCBK_Date_Helper()->get_time_sum( $this->from, 1 - $this->cancelled_duration, $this->cancelled_unit );
                        if ( $now < $last_cancel_date ) {
                            $value = true;
                        }
                    }
                    break;
                default:
                    $value = true;
            }

            return apply_filters( 'yith_wcbk_booking_can_be_' . $status, $value, $this );
        }

        /**
         * Checks the booking status against a passed in status.
         *
         * @return bool
         */
        public function has_status( $status ) {
            return apply_filters( 'yith_wcbk_booking_has_status', ( is_array( $status ) && in_array( $this->get_status(), $status ) ) || $this->get_status() === $status ? true : false, $this, $status );
        }

        /**
         * Updates status of booking
         *
         * @param string $new_status
         * @param string $activity
         * @param string $additional_note
         *
         */
        public function update_status( $new_status, $activity = 'change_status', $additional_note = '' ) {
            if ( !$this->id ) {
                return;
            }

            // Standardise status names.
            $new_status = 'bk-' === substr( $new_status, 0, 3 ) ? substr( $new_status, 3 ) : $new_status;
            $old_status = $this->get_status();

            $edited_by = '';

            if ( $new_status !== $old_status && ( in_array( $new_status, array_keys( yith_wcbk_get_booking_statuses( true ) ) ) ) ) {

                switch ( $new_status ) {
                    case 'cancelled_by_user' :
                        if ( !$this->can_be( 'cancelled_by_user' ) )
                            return;
                        $this->set( 'status', 'cancelled' );
                        $edited_by = ' ' . __( 'by customer', 'yith-booking-for-woocommerce' );
                        break;
                    default:
                        if ( !$this->can_be( $new_status ) )
                            return;
                        $this->set( 'status', $new_status );
                        break;
                }

                $current_status = $new_status !== 'cancelled_by_user' ? $new_status : 'cancelled';
                $booking_note   = sprintf( __( 'Booking status updated to %s.', 'yith-booking-for-woocommerce' ), yith_wcbk_get_booking_status_name( $current_status ) . $edited_by );
                $booking_note   .= !!$additional_note ? ' ' . $additional_note : '';
                $this->add_note( 'status_changed', $booking_note );

                // Status was changed
                do_action( 'yith_wcbk_booking_status_' . $new_status, $this->id );
                do_action( 'yith_wcbk_booking_status_' . $old_status . '_to_' . $new_status, $this->id );
                do_action( 'yith_wcbk_booking_status_changed', $this->id, $old_status, $new_status );
            }

            //do_action( 'yith_wcmbs_delete_transients' );
        }
    }
}