<?php
/**
 * Product class
 *
 * @author  Yithemes
 * @package YITH Booking for WooCommerce Premium
 * @version 1.0.0
 */


if ( !defined( 'YITH_WCBK' ) ) {
    exit;
} // Exit if accessed directly

if ( !class_exists( 'WC_Product_Booking' ) ) {
    /**
     * WC Product Booking
     *
     * @since 1.0.0
     */
    class WC_Product_Booking extends WC_Product {

        private $_enabled_person_types = null;

        private $_booking_props = array();

        /**
         * Constructor gets the post object and sets the ID for the loaded product.
         *
         * @param int|WC_Product|object $product Product ID, post object, or product object
         */
        public function __construct( $product ) {
            $this->product_type = 'booking';
            $this->virtual      = 'yes';
            parent::__construct( $product );
        }

        /**
         * @param $key
         *
         * @since 1.1.0
         * @return mixed|void
         */
        public function get_booking_prop( $key ) {
            if ( !isset( $this->_booking_props[ $key ] ) ) {
                $value = apply_filters( 'yith_wcbk_booking_product_get_booking_prop', null, $key, $this );
                if ( !is_null( $value ) ) {
                    if ( false !== $value ) {
                        $this->_booking_props[ $key ] = $value;
                    }

                    return $value;
                }

                $booking_key                  = '_yith_booking_' . $key;
                $this->_booking_props[ $key ] = get_post_meta( $this->get_id(), $booking_key, true );
            }

            return $this->_booking_props[ $key ];
        }

        /**
         * @param $key
         * @param $value
         *
         * @since 1.1.0
         */
        public function set_booking_prop( $key, $value ) {
            $this->_booking_props[ $key ] = $value;
        }

        /**
         * @return string
         */
        public function get_type() {
            return 'booking';
        }

        /**
         * Get the add to cart button text
         *
         * @access public
         * @return string
         */
        public function add_to_cart_text() {
            return apply_filters( 'woocommerce_product_add_to_cart_text', yith_wcbk_get_label( 'read-more' ), $this );
        }

        /**
         * Calculate price for Booking product
         *
         * @param array $args
         *
         * @return string price
         */
        public function calculate_price( $args = array() ) {
            /* NOTE: Duration is the number of booked blocks! */

            $from = isset( $args[ 'from' ] ) ? $args[ 'from' ] : time();
            $from = !is_numeric( $from ) ? strtotime( $from ) : $from;

            $duration      = isset( $args[ 'duration' ] ) ? absint( $args[ 'duration' ] ) : 1;
            $person_number = isset( $args[ 'persons' ] ) ? $args[ 'persons' ] : $this->get_min_persons();
            if ( !empty( $args[ 'person_types' ] ) && is_array( $args[ 'person_types' ] ) && $this->has_persons() ) {
                $person_number = 0;
                foreach ( $args[ 'person_types' ] as $person_type ) {
                    $persons       = absint( $person_type[ 'number' ] );
                    $person_number += $persons;
                }
                // set the total number of persons to allow discount/increment based on total person number
                $args[ 'persons' ] = $person_number;
            }

            $date_helper = YITH_WCBK_Date_Helper();

            if ( isset( $args[ 'to' ] ) ) {
                $to = $args[ 'to' ];
                $to = !is_numeric( $to ) ? strtotime( $to ) : $to;

                $duration           = $date_helper->get_time_diff( $from, $to, $this->get_duration_unit() ) / $this->get_booking_prop( 'duration' );
                $args[ 'duration' ] = $duration;
            }

            $person_types = array(
                array(
                    'id'     => 0,
                    'number' => $person_number,
                ),
            );

            if ( !empty( $args[ 'person_types' ] ) && $this->has_persons() && $this->has_multiply_costs_by_persons_enabled() ) {
                $person_types = $args[ 'person_types' ];
            }

            $price = 0;

            $booking_services = isset( $args[ 'booking_services' ] ) ? $args[ 'booking_services' ] : array();


            foreach ( $person_types as $person_type ) {
                // base cost depends only from starting date
                $args[ 'person_type' ] = $person_type;
                $person_type_number    = absint( $person_type[ 'number' ] );

                $base_cost             = $this->calculate_cost( $from, 'base', $args );
                $unit                  = $this->get_duration_unit();
                $single_duration_block = $this->get_booking_prop( 'duration' );

                $block_cost = 0;

                // increase the block cost for every block in base of settings
                for ( $b = 0; $b < $duration; $b++ ) {
                    $referring_date = $date_helper->get_time_sum( $from, $single_duration_block * $b, $unit, true );
                    $block_cost     += $this->calculate_cost( $referring_date, 'block', $args );
                }

                // total price
                $single_price = $base_cost + $block_cost;

                // calculate costs in base of Persons
                if ( $this->has_persons() && $this->has_multiply_costs_by_persons_enabled() ) {
                    $single_price = $single_price * $person_type_number;
                }

                $price += $single_price;
            }

            $service_args  = array(
                'persons'          => $person_number,
                'person_types'     => !empty( $args[ 'person_types' ] ) ? $args[ 'person_types' ] : $person_types,
                'duration'         => $duration,
                'booking_services' => $booking_services,
            );
            $service_costs = $this->calculate_service_costs( $service_args );

            $price += floatval( $service_costs );

            return (string) $price;
        }


        /**
         * Calculate the total service costs
         *
         * @param array $args
         *
         * @return string
         */
        public function calculate_service_costs( $args = array() ) {
            extract( $args );
            /**
             * @var int   $persons
             * @var array $person_types
             * @var int   $duration
             * @var array $booking_services
             */
            $service_cost = 0;

            if ( $persons > 0 && $this->has_services() ) {
                $services = $this->get_services();
                foreach ( $services as $service_id ) {
                    $service = yith_get_booking_service( $service_id );

                    if ( !$service->is_valid() )
                        continue;

                    if ( $service->is_optional() && !in_array( $service_id, $booking_services ) ) {
                        continue;
                    }

                    $service_cost_total = 0;

                    if ( $service->is_multiply_per_persons() ) {
                        foreach ( $person_types as $person_type ) {
                            $pt_id                      = $person_type[ 'id' ];
                            $pt_number                  = $person_type[ 'number' ];
                            $current_service_cost_total = $service->get_price( $pt_id );

                            if ( $current_service_cost_total > 0 ) {
                                if ( $service->is_multiply_per_blocks() ) {
                                    $current_service_cost_total *= $duration;
                                }
                                if ( $service->is_multiply_per_persons() ) {
                                    $current_service_cost_total *= $pt_number;
                                }
                            }

                            $service_cost_total += floatval( $current_service_cost_total );
                        }
                    } else {
                        $service_cost_total = $service->get_price();
                        if ( $service_cost_total > 0 ) {
                            if ( $service->is_multiply_per_blocks() ) {
                                $service_cost_total *= $duration;
                            }
                        }
                    }

                    $service_cost_total = apply_filters( 'yith_wcbk_booking_product_single_service_cost_total', $service_cost_total, $service, $args, $this );

                    $service_cost += floatval( $service_cost_total );
                }
            }

            $service_cost = apply_filters( 'yith_wcbk_booking_product_calculate_service_costs', $service_cost, $args, $this );

            return (string) $service_cost;
        }

        /**
         * Return the service cost in base of duration(number of blocks), person type and person number
         *
         * @param int   $duration
         * @param int   $person_type_id
         * @param int   $person_number
         *
         * @param array $optional_services_selected
         *
         * @return int|string
         */
        public function calculate_service_cost( $duration, $person_type_id, $person_number, $optional_services_selected = array() ) {
            $service_cost = 0;
            if ( $person_number > 0 && $this->has_services() ) {
                $services = $this->get_services();
                foreach ( $services as $service_id ) {
                    $service = yith_get_booking_service( $service_id );

                    if ( !$service->is_valid() )
                        continue;

                    if ( $service->is_optional() && !in_array( $service_id, $optional_services_selected ) ) {
                        continue;
                    }

                    $current_service_cost = $service->get_price( $person_type_id );

                    if ( $current_service_cost > 0 ) {
                        if ( $service->is_multiply_per_blocks() ) {
                            $current_service_cost *= $duration;
                        }
                        if ( $service->is_multiply_per_persons() ) {
                            $current_service_cost *= $person_number;
                        }
                    }

                    $service_cost += $current_service_cost;
                }
            }

            return $service_cost;
        }

        /**
         * Calulates costs (block or base) for a single timestamp
         *
         * @param        $timestamp
         * @param string $type
         * @param array  $args {
         *
         * @var array person_type the type of person ['id' => id, ...]
         * }
         *
         * @return float|mixed
         */
        public function calculate_cost( $timestamp, $type = 'block', $args = array() ) {
            do_action( 'yith_wcbk_booking_before_calculate_cost', $timestamp, $type, $this );
            $allowed_types = array( 'base', 'block' );
            $type          = in_array( $type, $allowed_types ) ? $type : 'block';
            $cost_variable = $type . '_cost';

            $cost        = $this->get_booking_prop( $cost_variable );
            $cost_ranges = $this->get_costs_ranges();

            $global_cost_ranges = YITH_WCBK()->settings->get_global_cost_ranges();
            $cost_ranges        = array_merge( $cost_ranges, $global_cost_ranges );


            if ( isset( $args[ 'person_type' ] ) ) {
                $current_person_type = $args[ 'person_type' ];
                $person_types        = $this->get_person_types();
                if ( $current_person_type[ 'id' ] !== 0 && isset( $person_types[ $current_person_type[ 'id' ] ] ) ) {
                    $product_person_type = $person_types[ $current_person_type[ 'id' ] ];
                    if ( $product_person_type[ $type . '_cost' ] !== '' ) {
                        $cost = $product_person_type[ $type . '_cost' ];
                    }
                }
            }

            $date_helper = YITH_WCBK_Date_Helper();

            $no_date_ranges = array( 'person', 'block' );

            $persons  = isset( $args[ 'persons' ] ) ? absint( $args[ 'persons' ] ) : $this->get_booking_prop( 'min_persons' );
            $duration = isset( $args[ 'duration' ] ) ? absint( $args[ 'duration' ] ) : 1;

            foreach ( $cost_ranges as $cost_range ) {
                $cost_range = (array) $cost_range;
                $date_from  = $timestamp;
                $date_to    = $timestamp;
                $range_type = !empty( $cost_range[ 'type' ] ) ? $cost_range[ 'type' ] : 'custom';
                $range_from = !empty( $cost_range[ 'from' ] ) ? $cost_range[ 'from' ] : '';
                $range_to   = !empty( $cost_range[ 'to' ] ) ? $cost_range[ 'to' ] : '';
                $intersect  = false;

                if ( !in_array( $range_type, $no_date_ranges ) ) {
                    $check = $date_helper->check_date_inclusion_in_range( $range_type, $range_from, $range_to, $date_from, $date_to, $intersect );
                } else {
                    $check = false;
                    // Calculate price in base of person number
                    if ( $cost_range[ 'type' ] === 'person' && $this->has_persons() ) {
                        if ( $persons <= $range_to && $persons >= $range_from ) {
                            $check = true;
                        }
                    } elseif ( $cost_range[ 'type' ] === 'block' ) {
                        // Calculate price in base of block number
                        // Note: duration is the number of blocks
                        if ( $duration <= $range_to && $duration >= $range_from ) {
                            $check = true;
                        }
                    }
                }

                $check = apply_filters( 'yith_wcbk_booking_calculate_cost_check_is_in_range', $check, $cost_range, $timestamp, $type, $this );

                if ( $check ) {
                    $this_cost     = !empty( $cost_range[ $type . '_cost' ] ) ? $cost_range[ $type . '_cost' ] : 0;
                    $this_operator = !empty( $cost_range[ $type . '_cost_operator' ] ) ? $cost_range[ $type . '_cost_operator' ] : 'add';
                    switch ( $this_operator ) {
                        case 'add':
                            $cost = $cost + $this_cost;
                            break;
                        case 'sub':
                            $cost = $cost - $this_cost;
                            break;
                        case 'mul':
                            $cost = $cost * $this_cost;
                            break;
                        case 'div':
                            if ( $this_cost != 0 )
                                $cost = $cost / $this_cost;
                            break;
                    }
                }
            }

            $cost = apply_filters( 'yith_wcbk_booking_calculate_cost', $cost, $timestamp, $type, $this );

            return $cost;

        }

        /**
         * Get the availability range array
         *
         * @return array
         */
        public function get_availability_ranges() {
            $availability_ranges = $this->get_booking_prop( 'availability_range' );

            return !empty( $availability_ranges ) ? $availability_ranges : array();
        }

        /**
         * Retrieve an array containing the booking data
         *
         * @return array
         */
        public function get_booking_data() {
            $booking_data_vars = apply_filters( 'yith_wcbk_get_booking_data_vars', array(
                'min_persons',
                'max_persons'
            ), $this );

            $booking_data = array();
            foreach ( $booking_data_vars as $var ) {
                $booking_data[ $var ] = $this->get_booking_prop( $var );
            }

            return apply_filters( 'yith_wcbk_get_booking_data', $booking_data, $this );
        }

        /**
         * Get the costs range array
         *
         * @return array
         */
        public function get_costs_ranges() {
            $costs_ranges = $this->get_booking_prop( 'costs_range' );

            return !!$costs_ranges && is_array( $costs_ranges ) ? $costs_ranges : array();
        }

        /**
         * Returns the product's base cost,
         * calculated by (base cost + booking cost) * persons .
         *
         * @return string price
         */
        public function get_base_cost() {
            $base_cost = $this->get_booking_prop( 'base_cost' ) + ( $this->get_booking_prop( 'block_cost' ) * $this->get_min_duration() );

            if ( $this->has_persons() && $this->get_min_persons() > 1 && $this->has_multiply_costs_by_persons_enabled() ) {
                $base_cost = $base_cost * $this->get_min_persons();
            }

            return $base_cost;
        }

        /**
         * get the block duration
         */
        public function get_block_duration_html() {
            $duration = $this->get_booking_prop( 'duration' );

            return $duration . ' ' . yith_wcbk_get_duration_unit_label( $this->get_duration_unit(), $duration );
        }

        /**
         * Get the calculated price html
         *
         * @param array $args
         *
         * @return string
         */
        public function get_calculated_price_html( $args = array() ) {
            $price = yit_get_display_price( $this, $this->calculate_price( $args ) );

            if ( $price == 0 ) {
                $price = apply_filters( 'yith_wcbk_booking_product_free_price_html', __( 'Free!', 'woocommerce' ), $this );
            } else {
                $price = wc_price( $price ) . $this->get_price_suffix();
            }

            return apply_filters( 'woocommerce_get_price_html', $price, $this );
        }

        /**
         * return the default start date
         *
         * @since 1.0.6
         *
         * @return string
         */
        public function get_default_start_date() {
            $date = '';
            switch ( $this->get_booking_prop( 'default_start_date' ) ) {
                case 'today':
                    $date = date( 'Y-m-d', strtotime( 'now' ) );
                    break;
                case 'tomorrow':
                    $date = date( 'Y-m-d', strtotime( 'tomorrow' ) );
                    break;
                case 'custom':
                    $date = $this->get_booking_prop( 'default_start_date_custom' );
                    break;
            }

            return apply_filters( 'yith_wcbk_booking_product_get_default_start_date', $date, $this );

        }

        /**
         * Get the duration unit
         *
         * @return string
         */
        public function get_duration_unit() {
            return $this->get_booking_prop( 'duration_unit' );
        }

        /**
         * Get the location coordinates
         *
         * @return array|bool
         */
        public function get_location_coordinates() {
            $coordinates = false;

            $lat = $this->get_booking_prop( 'location_lat' );
            $lng = $this->get_booking_prop( 'location_lng' );

            if ( $lat != '' && $lng != '' ) {
                $coordinates = array(
                    'lat' => $lat,
                    'lng' => $lng,
                );
            } elseif ( $location = $this->get_booking_prop( 'location' ) ) {
                $location_info = YITH_WCBK()->maps->get_location_by_address( $location );
                if ( $location_info && isset( $location_info[ 'lat' ] ) && isset( $location_info[ 'lng' ] ) ) {
                    $coordinates = $location_info;
                }

            }

            return $coordinates;
        }

        /**
         * Get the maximum duration
         *
         * @return int
         */
        public function get_max_duration() {
            if ( !$this->is_type_fixed_blocks() ) {
                return max( 0, absint( $this->get_booking_prop( 'maximum_duration' ) ) );
            } else {
                return 0;
            }
        }

        /**
         * Get the maximum duration time object
         *
         * @return object
         */
        public function get_max_duration_time() {
            $duration = $this->get_max_duration() * $this->get_booking_prop( 'duration' );
            $unit     = $this->get_duration_unit();
            $r        = array(
                'duration' => $duration,
                'unit'     => $unit,
            );

            return (object) $r;
        }

        /**
         * Get the minimum duration
         *
         * @return int
         */
        public function get_min_duration() {
            if ( !$this->is_type_fixed_blocks() ) {
                return max( 1, absint( $this->get_booking_prop( 'minimum_duration' ) ) );
            } else {
                return 1;
            }
        }

        /**
         * Get the minimum duration time object
         *
         * @return object
         */
        public function get_min_duration_time() {
            $duration = $this->get_min_duration() * $this->get_booking_prop( 'duration' );
            $unit     = $this->get_duration_unit();
            $r        = array(
                'duration' => $duration,
                'unit'     => $unit,
            );

            return (object) $r;
        }

        /**
         * Get the minimum nomber of persons
         *
         * @return int
         */
        public function get_min_persons() {
            return absint( $this->get_booking_prop( 'min_persons' ) );
        }

        /**
         * Returns the product's active price.
         *
         * @param string $context
         *
         * @return string price
         */
        public function get_price( $context = 'view' ) {
            $price = $this instanceof WC_Data ? parent::get_price( 'edit' ) : $this->price;
            $price = $price ? $price : $this->calculate_price();

            return 'view' === $context ? apply_filters( 'woocommerce_get_price', $price, $this ) : $price;
        }

        /**
         * Returns the product's regular price.
         * In case of Booking Product the regular price is ''
         *
         * @param string $context
         *
         * @return string price
         */
        public function get_regular_price( $context = 'view' ) {
            return '';
        }

        /**
         * Checks if a product has the calendar picker enabled
         *
         * @return bool
         */
        public function has_calendar_picker_enabled() {
            return $this->get_booking_prop( 'enable_calendar_range_picker' ) === 'yes' &&
                   $this->get_booking_prop( 'duration' ) == 1 &&
                   $this->get_duration_unit() === 'day' &&
                   !$this->is_type_fixed_blocks();
        }

        /**
         * Checks if a product has multiply costs by persons enabled.
         *
         * @return bool
         */
        public function has_multiply_costs_by_persons_enabled() {
            return $this->has_persons() && $this->get_booking_prop( 'multiply_costs_by_persons' ) === 'yes';
        }

        /**
         * Check if has persons enabled.
         *
         * @return boolean
         */
        public function has_persons() {
            return $this->get_booking_prop( 'has_persons' ) === 'yes';
        }

        /**
         * Check if has person types enabled.
         *
         * @return boolean
         */
        public function has_person_types() {
            $person_type_enabled = $this->get_booking_prop( 'enable_person_types' ) === 'yes';
            $person_types        = $this->get_person_types();

            return $person_type_enabled && !!$person_types;
        }

        /**
         * get the person types
         *
         * @return array
         */
        public function get_person_types() {
            if ( is_null( $this->_enabled_person_types ) ) {
                $this->_enabled_person_types = array();
                $person_types                = !!$this->get_booking_prop( 'person_types' ) && is_array( $this->get_booking_prop( 'person_types' ) ) ? $this->get_booking_prop( 'person_types' ) : array();
                foreach ( $person_types as $person_type_id => $person_type ) {
                    $person_type[ 'enabled' ] = isset( $person_type[ 'enabled' ] ) && $person_type[ 'enabled' ] === 'yes';
                    if ( $person_type[ 'enabled' ] ) {
                        $this->_enabled_person_types[ $person_type_id ] = $person_type;
                    }
                }
            }

            return $this->_enabled_person_types;
        }

        /**
         * get the services
         *
         * @param array $args arguments passed to wp_get_object_terms
         *
         * @return array
         */
        public function get_services( $args = array() ) {
            if ( !!$this->get_booking_prop( 'services' ) ) {
                return $this->get_booking_prop( 'services' );
            }

            $default_args = array(
                'fields' => 'ids',
            );

            $args               = wp_parse_args( $args, $default_args );
            $booking_product_id = apply_filters( 'yith_wcbk_booking_product_id_to_translate', $this->get_id(), $this );
            $services           = wp_get_object_terms( $booking_product_id, YITH_WCBK_Post_Types::$service_tax, $args );
            $services           = apply_filters( 'yith_wcbk_booking_product_get_services', $services, $this );

            $this->set_booking_prop( 'services', $services );

            return $this->get_booking_prop( 'services' );
        }

        /**
         * get valid availability range types in base of duration unit of the booking product
         *
         * @return array
         */
        public function get_valid_availability_range_types() {
            $valid_ranges = array(
                'custom',
                'month',
                'day',
                'all-week',
                'day-1',
                'day-2',
                'day-3',
                'day-4',
                'day-5',
                'day-6',
                'day-7',
            );
            $unit         = $this->get_duration_unit();
            if ( in_array( $unit, array( 'month', 'day' ) ) ) {
                $max_index    = array_search( $unit, $valid_ranges ) + 1;
                $valid_ranges = array_slice( $valid_ranges, 0, $max_index );
            }

            return $valid_ranges;
        }

        /**
         * check if this booking has services
         *
         * @return array
         */
        public function has_services() {
            return !!$this->get_services();
        }

        /**
         * Check if this booking is available
         *
         * @param array $args {
         *
         * @var int     $from [optional] timestamp from date
         * @var int     $to [optional] timestamp to date
         * }
         *
         * @return bool
         */
        public function is_available( $args = array() ) {
            do_action( 'yith_wcbk_booking_before_is_available', $args, $this );

            $date_helper = YITH_WCBK_Date_Helper();

            $from          = isset( $args[ 'from' ] ) ? $args[ 'from' ] : time();
            $to            = isset( $args[ 'to' ] ) ? $args[ 'to' ] : $from;
            $exclude_hours = isset( $args[ 'exclude_hours' ] ) ? $args[ 'exclude_hours' ] : false;

            $valid_range_types = $this->get_valid_availability_range_types();

            $min_duration_time = $this->get_min_duration_time();
            $max_duration_time = $this->get_max_duration_time();
            if ( empty( $args[ 'to' ] ) ) {


                $to = $date_helper->get_time_sum( $from, $min_duration_time->duration, $min_duration_time->unit );
            } else {
                $min_to = $date_helper->get_time_sum( $from, $min_duration_time->duration, $min_duration_time->unit, true );

                if ( $to < $min_to ) {
                    return false;
                }

                if ( $max_duration_time->duration > 0 ) {
                    $max_to = $date_helper->get_time_sum( $from, $max_duration_time->duration, $max_duration_time->unit, true );
                    if ( $to > $max_to )
                        return false;
                }
            }

            // Check if booking is available depending on Global availability settings and Product availability settings
            $available = $this->check_availability( $from, $to );

            // Check if exist other booked booking (for the same product) in the same dates!
            if ( $available ) {
                $get_post_args = array();
                if ( isset( $args[ '_booking_id' ] ) ) {
                    // exclude the booking if the customer is paying for a his/her confirmed booking
                    $booking_id    = apply_filters( 'yith_wcbk_booking_product_id_to_translate', absint( $args[ '_booking_id' ] ) );
                    $get_post_args = array(
                        'exclude' => $booking_id,
                    );
                }
                $booking_id            = apply_filters( 'yith_wcbk_booking_product_id_to_translate', $this->get_id() );
                $number_of_bookings    = YITH_WCBK_Booking_Helper()->count_booked_bookings_in_period( $booking_id, $from, $to, $get_post_args );
                $max_booking_per_block = $this->get_booking_prop( 'max_per_block' );
                $max_booking_per_block = $max_booking_per_block > 1 ? $max_booking_per_block : 1;
                if ( $number_of_bookings >= $max_booking_per_block ) {
                    $available = false;
                }
            }

            $available = apply_filters( 'yith_wcbk_booking_is_available', $available, $args, $this );

            return $available;
        }


        public function check_availability( $from, $to ) {
            $date_helper       = YITH_WCBK_Date_Helper();
            $available         = true;
            $valid_range_types = $this->get_valid_availability_range_types();

            $availability_range = YITH_WCBK()->settings->get_global_availability_ranges();
            $availability_range = array_merge( $availability_range, (array) $this->get_availability_ranges() );

            foreach ( $availability_range as $range ) {
                $tmp_from = $from;
                $tmp_to   = $to - 1; // subtract one second to fix days and months availability (include the last rule day)

                $range_bookable = $range->bookable === 'yes';

                $check = $date_helper->check_date_inclusion_in_range( $range->type, $range->from, $range->to, $tmp_from, $tmp_to, !$range_bookable );
                $check = apply_filters( 'yith_wcbk_booking_is_available_check_is_in_range', $check, $range, $tmp_from, $tmp_to, $available, $this );

                if ( $check && in_array( 'day', $valid_range_types ) && $range->days_enabled && !!$range->days ) {
                    $range_bookable = true;
                    foreach ( $range->days as $day_number => $day_bookable ) {
                        $day_bookable = $day_bookable === 'yes';
                        $day_check    = $date_helper->check_date_inclusion_in_range( 'day', $day_number, $day_number, $tmp_from, $tmp_to, !$day_bookable );
                        if ( $day_check )
                            $range_bookable = $range_bookable && $day_bookable;
                    }
                }

                if ( $check )
                    $available = $range_bookable;
            }

            return $available;

        }

        /**
         * Check if the duration type is "Fixed blocks"
         *
         * @return boolean
         */
        public function is_type_fixed_blocks() {
            return 'fixed' === $this->get_booking_prop( 'duration_type' );
        }

        /**
         * Returns false if the product cannot be bought.
         *
         * @return bool
         */
        public function is_purchasable() {
            $purchasable = true;

            // Products must exist of course
            if ( !$this->exists() ) {
                $purchasable = false;

                // Check the product is published
            } elseif ( ( $this instanceof WC_Data ? $this->get_status() : $this->post->post_status ) !== 'publish' && !current_user_can( 'edit_post', $this->get_id() ) ) {
                $purchasable = false;
            }

            return apply_filters( 'woocommerce_is_purchasable', $purchasable, $this );
        }

        /**
         * Check if Admin has to confirm before purchase booking
         *
         * @return boolean
         */
        public function is_requested_confirmation() {
            return 'yes' === $this->get_booking_prop( 'request_confirmation' );
        }

        /**
         * The booking product is sold individually
         *
         * @return boolean
         */
        public function is_sold_individually() {
            return true;
        }

        /**
         * Checks if a product is virtual (has no shipping).
         *
         * @return bool
         */
        public function is_virtual() {
            return true;
        }

        /**
         * @param int    $year
         * @param int    $month
         *
         * @param string $return
         * @param string $range day or month
         *
         * @return array
         */
        public function create_availability_month_calendar( $year = 0, $month = 0, $return = 'all', $range = 'day' ) {
            // default for year and month
            $year  = $year == 0 ? date( 'Y', time() ) : $year;
            $month = $month == 0 ? date( 'm', time() ) : $month;

            $month_calendar = array();

            $first_day_of_month      = strtotime( $year . '-' . $month . '-01' );
            $first_day_of_next_month = strtotime( '+1 month', $first_day_of_month );

            $current_day = $first_day_of_month;
            while ( $current_day < $first_day_of_next_month ) {
                $number_of_day = date( 'j', $current_day );
                switch ( $range ) {
                    case 'month':
                        $next_day = $first_day_of_next_month;
                        break;
                    case 'day':
                    default:
                        $next_day = strtotime( '+1 day', $current_day );
                }

                $is_available = $this->is_available( array( 'from' => $current_day, 'exclude_hours' => true ) );

                switch ( $return ) {
                    case 'bookable':
                        if ( $is_available ) {
                            $month_calendar[ $number_of_day ] = $is_available;
                        }
                        break;
                    case 'not_bookable':
                        if ( !$is_available ) {
                            $month_calendar[ $number_of_day ] = $is_available;
                        }
                        break;
                    default:
                        $month_calendar[ $number_of_day ] = $is_available;

                }
                $current_day = $next_day;
            }

            return $month_calendar;
        }

        /**
         * @param int    $year
         *
         * @param int    $from_month
         * @param int    $to_month
         *
         * @param string $return
         * @param string $range
         *
         * @return array
         */
        public function create_availability_year_calendar( $year = 0, $from_month = 1, $to_month = 12, $return = 'all', $range = 'day' ) {
            $year_calendar = array();
            for ( $i = $from_month; $i <= $to_month; $i++ ) {
                $this_month_calendar = $this->create_availability_month_calendar( $year, $i, $return, $range );
                if ( !empty( $this_month_calendar ) )
                    $year_calendar[ $i ] = $this->create_availability_month_calendar( $year, $i, $return, $range );
            }

            return $year_calendar;
        }

        /**
         * @param        $from_year
         * @param        $from_month
         * @param        $to_year
         * @param        $to_month
         *
         * @param string $return
         * @param string $range
         *
         * @return array
         */
        public function create_availability_calendar( $from_year, $from_month, $to_year, $to_month, $return = 'all', $range = 'day' ) {
            $calendar = array();

            for ( $year = $from_year; $year <= $to_year; $year++ ) {
                $first_month        = $year == $from_year ? $from_month : 1;
                $last_month         = $year == $to_year ? $to_month : 12;
                $this_year_calendar = $this->create_availability_year_calendar( $year, $first_month, $last_month, $return, $range );
                if ( !empty( $this_year_calendar ) )
                    $calendar[ $year ] = $this_year_calendar;
            }

            return $calendar;
        }

        /**
         * @param        $from_year
         * @param        $from_month
         * @param        $to_year
         * @param        $to_month
         *
         * @param string $range
         *
         * @return array
         */
        public function get_not_available_dates( $from_year, $from_month, $to_year, $to_month, $range = 'day' ) {
            $calendar = $this->create_availability_calendar( $from_year, $from_month, $to_year, $to_month, 'not_bookable', $range );
            $dates    = array();
            foreach ( $calendar as $year => $months ) {
                foreach ( $months as $month => $days ) {
                    if ( $month < 10 ) {
                        $month = '0' . $month;
                    }
                    foreach ( $days as $day => $bookable ) {
                        if ( $day < 10 ) {
                            $day = '0' . $day;
                        }
                        $dates[] = $year . '-' . $month . '-' . $day;
                    }
                }
            }

            return $dates;
        }

        /**
         * @param        $from_year
         * @param        $from_month
         * @param        $to_year
         * @param        $to_month
         *
         * @return array
         */
        public function get_not_available_months( $from_year, $from_month, $to_year, $to_month ) {
            $dates           = $this->get_not_available_dates( $from_year, $from_month, $to_year, $to_month, 'month' );
            $number_of_dates = count( $dates );
            if ( $number_of_dates < 1 ) {
                return array();
            }

            $zero_array  = array_fill( 0, $number_of_dates, 0 );
            $seven_array = array_fill( 0, $number_of_dates, 7 );
            $dates       = array_map( 'substr', $dates, $zero_array, $seven_array );

            return $dates;
        }

        /**
         * Get the add to cart button text for the single page.
         *
         * @return string
         */
        public function single_add_to_cart_text() {
            $booking_single_add_to_cart_text = !$this->is_requested_confirmation() ? yith_wcbk_get_label( 'add-to-cart' ) : yith_wcbk_get_label( 'request-confirmation' );

            return apply_filters( 'woocommerce_product_single_add_to_cart_text', $booking_single_add_to_cart_text, $this );
        }
    }
}
