<?php
/**
 * Service class
 *
 * @author      Yithemes
 * @package     YITH Booking for WooCommerce Premium
 * @version     1.0.0
 */


if ( !defined( 'YITH_WCBK' ) ) {
    exit;
} // Exit if accessed directly

if ( !class_exists( 'YITH_WCBK_Service' ) ) {
    /**
     * YITH_WCBK_Service
     *
     * @class       YITH_WCBK_Service
     *
     *
     * @property    string $name
     * @property    string $slug
     * @property    string $description
     * @property    string $price
     * @property    string $optional
     * @property    string $hidden
     * @property    string $multiply_per_blocks
     * @property    string $multiply_per_persons
     * @property    array  $price_for_person_types
     * @since       1.0.0
     */
    class YITH_WCBK_Service {

        /**
         * the id of the term
         *
         * @var int
         */
        public $id;

        /**
         * @var WP_Term
         */
        public $term;
        
        /**
         * the service taxonomy name
         *
         * @var string
         * @since 1.0.0
         */
        public $taxonomy_name;

        /**
         * Constructor
         *
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com>
         */
        public function __construct( $term_id, $term = null ) {
            $this->taxonomy_name = YITH_WCBK_Post_Types::$service_tax;
            $this->id            = absint( $term_id );

            $this->_populate( $term );
        }


        /**
         * __get function.
         *
         * @param string $key
         *
         * @return mixed
         */
        public function __get( $key ) {
            $value = apply_filters( 'yith_wcbk_booking_service_get', null, $key, $this );
            if ( is_null( $value ) )
                $value = get_term_meta( $this->id, $key, true );

            if ( !empty( $value ) ) {
                $this->$key = $value;
            }

            return $value;
        }

        /**
         * __isset function.
         *
         * @param string $key
         *
         * @return mixed
         */
        public function __isset( $key ) {
            return metadata_exists( 'term', $this->id, $key );
        }

        /**
         * __set function.
         *
         * @param string $property
         * @param mixed  $value
         *
         * @return bool|int
         */
        public function set( $property, $value ) {
            $this->$property = $value;

            return update_term_meta( $this->id, $property, wc_clean( $value ) );
        }

        /**
         * Get data of the service
         *
         * @param WP_Term $term
         */
        private function _populate( $term = null ) {
            if ( empty( $term ) ) {
                $this->term = get_term( $this->id, $this->taxonomy_name );
            } else {
                $this->term = $term;
            }
            if ( $this->is_valid() ) {
                $this->name        = $this->term->name;
                $this->description = $this->term->description;
                $this->slug        = $this->term->slug;

                foreach ( $this->get_meta() as $key => $value ) {
                    $this->$key = $value;
                }

                do_action( 'yith_wcbk_booking_service_loaded', $this );
            }
        }

        /**
         * check if the service is valid
         *
         * @return bool
         */
        public function is_valid() {
            return !empty( $this->term ) && !empty( $this->id ) && $this->term->term_id == $this->id;
        }


        /**
         * check if the service is hidden
         *
         * @return bool
         */
        public function is_hidden() {
            return 'yes' === $this->hidden;
        }

        /**
         * check if the service has multiply per blocks enabled
         *
         * @return bool
         */
        public function is_multiply_per_blocks() {
            return 'yes' === $this->multiply_per_blocks;
        }

        /**
         * check if the service has multiply per persons enabled
         *
         * @return bool
         */
        public function is_multiply_per_persons() {
            return 'yes' === $this->multiply_per_persons;
        }

        /**
         * check if the service is optional
         *
         * @return bool
         */
        public function is_optional() {
            return 'yes' === $this->optional;
        }


        /**
         * get the price of the current service
         *
         * @param int $person_type
         *
         * @return string
         */
        public function get_price_for_person_type( $person_type ) {
            $price = '';
            if ( $person_type ) {
                $price_for_person_types = $this->price_for_person_types;
                if ( isset( $price_for_person_types[ $person_type ] ) ) {
                    $price = $price_for_person_types[ $person_type ];
                }
            }

            return $price;
        }

        /**
         * get the price of the current service
         *
         * @param int $person_type
         *
         * @return string
         */
        public function get_price( $person_type = 0 ) {
            $price = $this->price;
            if ( $person_type ) {
                $price_for_person_type = $this->get_price_for_person_type( $person_type );
                if ( $price_for_person_type != '' ) {
                    $price = absint( $price_for_person_type );
                }
            }

            return $price;
        }

        /**
         * return the name of the service
         *
         * @return string
         */
        public function get_name() {
            return apply_filters( 'yith_wcbk_get_service_name', $this->name, $this );
        }

        /**
         * get the price HTML of the current service
         *
         * @param int $person_type
         *
         * @return string
         */
        public function get_price_html( $person_type = 0 ) {
            return wc_price( $this->get_price( $person_type ) );
        }

        /**
         * Fill the default metadata with the post meta stored in db
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com
         * @return array
         */
        public function get_meta() {
            $meta = array();
            foreach ( self::get_default_meta_data() as $key => $value ) {
                $meta[ $key ] = $this->$key;
            }

            return $meta;
        }

        /**
         * Return an array of all custom fields
         *
         *
         * @since  1.0.0
         * @author Leanza Francesco <leanzafrancesco@gmail.com
         * @return array
         */
        public static function get_default_meta_data() {
            return array(
                'price'                  => '',
                'optional'               => 'no',
                'hidden'                 => 'no',
                'multiply_per_blocks'    => 'no',
                'multiply_per_persons'   => 'no',
                'price_for_person_types' => array()
            );
        }
    }
}

if ( !function_exists( 'yith_get_booking_service' ) ) {
    function yith_get_booking_service( $service_id, $term = null ) {
        return new YITH_WCBK_Service( $service_id, $term );
    }
}