<?php
/**
 * Settings class
 *
 * @author  Yithemes
 * @package YITH Booking for WooCommerce Premium
 * @version 1.0.0
 */


if ( !defined( 'YITH_WCBK' ) ) {
    exit;
} // Exit if accessed directly

if ( !class_exists( 'YITH_WCBK_Settings' ) ) {
    /**
     * YITH Booking for WooCommerce Premium Settings
     *
     * @since 1.0.0
     */
    class YITH_WCBK_Settings {
        /**
         * Single instance of the class
         *
         * @var \YITH_WCBK_Settings
         * @since 1.0.0
         */
        protected static $_instance;

        /**
         * Returns single instance of the class
         *
         * @return \YITH_WCBK_Settings
         * @since 1.0.0
         */
        public static function get_instance() {
            if ( is_null( self::$_instance ) ) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }

        private function __construct() {
            if ( is_admin() ) {
                add_action( 'admin_init', array( $this, 'save_settings' ) );
            }
        }

        /**
         * Save Settings
         */
        public function save_settings() {
            if ( empty( $_POST[ 'yith_wcbk_nonce' ] ) || !wp_verify_nonce( $_POST[ 'yith_wcbk_nonce' ], 'yith_wcbk_settings_fields' ) ) {
                return;
            }

            if ( isset( $_POST[ 'yith_booking_global_availability_range' ] ) ) {
                $ranges         = $_POST[ 'yith_booking_global_availability_range' ];
                $type_array     = isset( $ranges[ 'type' ] ) ? $ranges[ 'type' ] : array();
                $from_array     = isset( $ranges[ 'from' ] ) ? $ranges[ 'from' ] : array();
                $to_array       = isset( $ranges[ 'to' ] ) ? $ranges[ 'to' ] : array();
                $bookable_array = isset( $ranges[ 'bookable' ] ) ? $ranges[ 'bookable' ] : array();
                $days_enabled   = isset( $ranges[ 'days_enabled' ] ) ? $ranges[ 'days_enabled' ] : array();
                $days           = isset( $ranges[ 'days' ] ) ? $ranges[ 'days' ] : array();

                $availability_size = sizeof( $type_array );
                $values            = array();
                for ( $i = 1; $i < $availability_size; $i++ ) {
                    if ( !empty( $type_array[ $i ] ) && !empty( $from_array[ $i ] ) && !empty( $to_array[ $i ] ) && !empty( $bookable_array[ $i ] ) ) {
                        $current_range = array(
                            'type'         => $type_array[ $i ],
                            'from'         => $from_array[ $i ],
                            'to'           => $to_array[ $i ],
                            'bookable'     => $bookable_array[ $i ],
                            'days_enabled' => $days_enabled[ $i ],
                            'days'         => array(),
                        );

                        if ( $days_enabled[ $i ] === 'yes' ) {
                            $current_range_days = array();
                            for ( $j = 1; $j <= 7; $j++ ) {
                                $current_range_days[ $j ] = isset( $days[ $j ][ $i ] ) ? $days[ $j ][ $i ] : $bookable_array[ $i ];
                            }

                            $current_range[ 'days' ] = $current_range_days;
                        }

                        $values[] = (object) $current_range;
                    }
                }

                update_option( 'yith_wcbk_booking_global_availability_ranges', $values );
            } elseif ( isset( $_POST[ 'yith_booking_global_cost_ranges' ] ) ) {
                $ranges              = $_POST[ 'yith_booking_global_cost_ranges' ];
                $type_array          = isset( $ranges[ 'type' ] ) ? $ranges[ 'type' ] : array();
                $from_array          = isset( $ranges[ 'from' ] ) ? $ranges[ 'from' ] : array();
                $to_array            = isset( $ranges[ 'to' ] ) ? $ranges[ 'to' ] : array();
                $base_cost_operator  = isset( $ranges[ 'base_cost_operator' ] ) ? $ranges[ 'base_cost_operator' ] : array();
                $base_cost           = isset( $ranges[ 'base_cost' ] ) ? $ranges[ 'base_cost' ] : array();
                $block_cost_operator = isset( $ranges[ 'block_cost_operator' ] ) ? $ranges[ 'block_cost_operator' ] : array();
                $block_cost          = isset( $ranges[ 'block_cost' ] ) ? $ranges[ 'block_cost' ] : array();

                $costs_size = sizeof( $type_array );
                $values     = array();
                for ( $i = 1; $i < $costs_size; $i++ ) {
                    if ( isset( $type_array[ $i ] ) && isset( $from_array[ $i ] ) && isset( $to_array[ $i ] ) && isset( $base_cost_operator[ $i ] ) && isset( $block_cost_operator[ $i ] ) ) {

                        $single_base_cost  = !empty( $base_cost[ $i ] ) ? $base_cost[ $i ] : '';
                        $single_block_cost = !empty( $block_cost[ $i ] ) ? $block_cost[ $i ] : '';

                        $values[] = (object) array(
                            'type'                => $type_array[ $i ],
                            'from'                => $from_array[ $i ],
                            'to'                  => $to_array[ $i ],
                            'base_cost_operator'  => $base_cost_operator[ $i ],
                            'base_cost'           => $single_base_cost,
                            'block_cost_operator' => $block_cost_operator[ $i ],
                            'block_cost'          => $single_block_cost,
                        );
                    }
                }

                update_option( 'yith_wcbk_booking_global_cost_ranges', $values );
            }
        }

        /**
         * retrieves the global availability range array
         *
         * @return array the array of ranges as StdClass
         */
        public function get_global_availability_ranges() {
            $ranges = get_option( 'yith_wcbk_booking_global_availability_ranges', array() );

            return !!$ranges && is_array( $ranges ) ? $ranges : array();
        }


        /**
         * retrieves the global cost range array
         *
         * @return array the array of ranges as StdClass
         */
        public function get_global_cost_ranges() {
            $ranges = get_option( 'yith_wcbk_booking_global_cost_ranges', array() );

            return !!$ranges && is_array( $ranges ) ? $ranges : array();
        }

        /**
         * get settings related to Booking plugin
         *
         * @param      $key
         * @param bool $default
         *
         * @return mixed|void
         */
        public function get( $key, $default = false ) {
            return get_option( 'yith-wcbk-' . $key, $default );
        }

        /**
         * return true if showing booking form requires login
         *
         * @return bool
         * @since 1.0.5
         */
        public function show_booking_form_to_logged_users_only() {
            $show = YITH_WCBK()->settings->get( 'show-booking-form-to-logged-users-only', 'no' ) === 'yes';

            return apply_filters( 'yith_wcbk_show_booking_form_to_logged_users_only', $show );
        }
    }
}